# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands.validators import validate_tag


def validate_storage_account_id(cmd, namespace):
    """Validate storage account name"""
    from azure.cli.core.commands.client_factory import get_subscription_id
    from msrestazure.tools import is_valid_resource_id, resource_id

    if namespace.storage_account:
        if not is_valid_resource_id(namespace.storage_account):
            namespace.storage_account = resource_id(
                subscription=get_subscription_id(cmd.cli_ctx),
                resource_group=namespace.resource_group_name,
                namespace='Microsoft.Storage', type='storageAccounts',
                name=namespace.storage_account
            )


def datetime_format(value):
    """Validate the correct format of a datetime string and deserialize."""
    from msrest.serialization import Deserializer
    from msrest.exceptions import DeserializationError

    try:
        datetime_obj = Deserializer.deserialize_iso(value)
    except DeserializationError:
        message = "Argument {} is not a valid ISO-8601 datetime format"
        raise ValueError(message.format(value))
    return datetime_obj


def validate_archive_window_length(ns):
    """Validate the correct format of a datetime string and the range."""
    if ns.archive_window_length is not None:
        from msrest.serialization import Deserializer
        from msrest.exceptions import DeserializationError

        try:
            datetime_obj = Deserializer.deserialize_duration(ns.archive_window_length)
        except DeserializationError:
            message = "archive-window-length {} is not a valid ISO-8601 duration format"
            raise ValueError(message.format(ns.archive_window_length))

        minwindow = Deserializer.deserialize_duration("PT5M")
        maxwindow = Deserializer.deserialize_duration("PT25H")

        if datetime_obj < minwindow or datetime_obj > maxwindow:
            message = "archive-window-length '{}' is not in the range of PT5M and PT25H"\
                .format(ns.archive_window_length)
            raise ValueError(message)


def validate_key_frame_interval_duration(ns):
    """Validate the correct format of a datetime string and the range."""
    if ns.key_frame_interval_duration is not None:
        from msrest.serialization import Deserializer
        from msrest.exceptions import DeserializationError

        try:
            datetime_obj = Deserializer.deserialize_duration(ns.key_frame_interval_duration)
        except DeserializationError:
            message = "key-frame-interval-duration {} is not a valid ISO-8601 duration format"
            raise ValueError(message.format(ns.key_frame_interval_duration))

        minwindow = Deserializer.deserialize_duration("PT1S")
        maxwindow = Deserializer.deserialize_duration("PT30S")

        if datetime_obj < minwindow or datetime_obj > maxwindow:
            message = "key-frame-interval-duration '{}' is not in the range of PT1S and PT30S"\
                .format(ns.key_frame_interval_duration)
            raise ValueError(message)


def validate_correlation_data(ns):
    """ Extracts multiple space-separated correlation data in key[=value] format """
    if isinstance(ns.correlation_data, list):
        correlation_data_dict = {}
        for item in ns.correlation_data:
            correlation_data_dict.update(validate_tag(item))
        ns.correlation_data = correlation_data_dict


def validate_token_claim(ns):
    """ Extracts multiple space-separated token claims in key[=value] format """
    if isinstance(ns.token_claims, list):
        token_claims_dict = {}
        for item in ns.token_claims:
            token_claims_dict.update(validate_tag(item))
        ns.token_claims = token_claims_dict


def validate_output_assets(ns):
    """ Extracts multiple space-separated output assets in key[=value] format """
    def _get_asset(asset_string):
        from azure.mgmt.media.models import JobOutputAsset

        name_and_label = asset_string.split('=')
        if len(name_and_label) <= 1:
            message = "Output assets are not in correct format. Output assets should be in 'assetName=label'" \
                      " format. An asset without label can be sent like this: 'assetName='"
            raise ValueError(message)
        name = name_and_label[0]
        label = name_and_label[1]
        return JobOutputAsset(asset_name=name, label=label)

    if isinstance(ns.output_assets, list):
        ns.output_assets = list(map(_get_asset, ns.output_assets))



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['dla'] = """
type: group
short-summary: Manage Data Lake Analytics accounts, jobs, and catalogs.
"""

helps['dla account'] = """
type: group
short-summary: Manage Data Lake Analytics accounts.
"""

helps['dla account blob-storage'] = """
type: group
short-summary: Manage links between Data Lake Analytics accounts and Azure Storage.
"""

helps['dla account blob-storage add'] = """
type: command
short-summary: Links an Azure Storage account to the specified Data Lake Analytics account.
"""

helps['dla account blob-storage update'] = """
type: command
short-summary: Updates an Azure Storage account linked to the specified Data Lake Analytics account.
"""

helps['dla account compute-policy'] = """
type: group
short-summary: Manage Data Lake Analytics account compute policies.
"""

helps['dla account compute-policy create'] = """
type: command
short-summary: Create a compute policy in the Data Lake Analytics account.
parameters:
  - name: --max-dop-per-job
    type: int
    short-summary: The maximum degree of parallelism allowed per job for this policy. At least one of `--min-priority-per-job` and `--max-dop-per-job` must be specified.
  - name: --min-priority-per-job
    type: int
    short-summary: The minimum priority allowed per job for this policy. At least one of `--min-priority-per-job` and `--max-dop-per-job` must be specified.
  - name: --compute-policy-name
    type: string
    short-summary: The name of the compute policy to create.
  - name: --object-id
    type: string
    short-summary: The Azure Active Directory object ID of the user, group, or service principal to apply the policy to.
  - name: --object-type
    type: string
    short-summary: The Azure Active Directory object type associated with the supplied object ID.
"""

helps['dla account compute-policy delete'] = """
type: command
short-summary: Delete a compute policy in a Data Lake Analytics account.
"""

helps['dla account compute-policy list'] = """
type: command
short-summary: List compute policies in the a Lake Analytics account.
"""

helps['dla account compute-policy show'] = """
type: command
short-summary: Retrieve a compute policy in a Data Lake Analytics account.
"""

helps['dla account compute-policy update'] = """
type: command
short-summary: Update a compute policy in the Data Lake Analytics account.
parameters:
  - name: --max-dop-per-job
    type: int
    short-summary: The maximum degree of parallelism allowed per job for this policy. At least one of `--min-priority-per-job` and `--max-dop-per-job` must be specified.
  - name: --min-priority-per-job
    type: int
    short-summary: The minimum priority allowed per job for this policy. At least one of `--min-priority-per-job` and `--max-dop-per-job` must be specified.
  - name: --compute-policy-name
    type: string
    short-summary: The name of the compute policy to update.
"""

helps['dla account create'] = """
type: command
short-summary: Create a Data Lake Analytics account.
parameters:
  - name: --default-data-lake-store
    type: string
    short-summary: The default Data Lake Store account to associate with the created account.
  - name: --max-degree-of-parallelism
    type: int
    short-summary: The maximum degree of parallelism for this account.
  - name: --max-job-count
    type: int
    short-summary: The maximum number of concurrent jobs for this account.
  - name: --query-store-retention
    type: int
    short-summary: The number of days to retain job metadata.
"""

helps['dla account data-lake-store'] = """
type: group
short-summary: Manage links between Data Lake Analytics and Data Lake Store accounts.
"""

helps['dla account delete'] = """
type: command
short-summary: Delete a Data Lake Analytics account.
"""

helps['dla account firewall'] = """
type: group
short-summary: Manage Data Lake Analytics account firewall rules.
"""

helps['dla account firewall create'] = """
type: command
short-summary: Create a firewall rule in a Data Lake Analytics account.
parameters:
  - name: --end-ip-address
    type: string
    short-summary: The end of the valid IP range for the firewall rule.
  - name: --start-ip-address
    type: string
    short-summary: The start of the valid IP range for the firewall rule.
  - name: --firewall-rule-name
    type: string
    short-summary: The name of the firewall rule.
"""

helps['dla account firewall delete'] = """
type: command
short-summary: Delete a firewall rule in a Data Lake Analytics account.
"""

helps['dla account firewall list'] = """
type: command
short-summary: List firewall rules in a Data Lake Analytics account.
"""

helps['dla account firewall show'] = """
type: command
short-summary: Retrieve a firewall rule in a Data Lake Analytics account.
"""

helps['dla account firewall update'] = """
type: command
short-summary: Update a firewall rule in a Data Lake Analytics account.
"""

helps['dla account list'] = """
type: command
short-summary: List available Data Lake Analytics accounts.
"""

helps['dla account show'] = """
type: command
short-summary: Get the details of a Data Lake Analytics account.
"""

helps['dla account update'] = """
type: command
short-summary: Update a Data Lake Analytics account.
parameters:
  - name: --max-degree-of-parallelism
    type: int
    short-summary: The maximum degree of parallelism for this account.
  - name: --max-job-count
    type: int
    short-summary: The maximum number of concurrent jobs for this account.
  - name: --query-store-retention
    type: int
    short-summary: The number of days to retain job metadata.
  - name: --firewall-state
    type: string
    short-summary: Enable or disable existing firewall rules.
  - name: --allow-azure-ips
    type: string
    short-summary: Allow or block IPs originating from Azure through the firewall.
"""

helps['dla catalog'] = """
type: group
short-summary: Manage Data Lake Analytics catalogs.
"""

helps['dla catalog assembly'] = """
type: group
short-summary: Manage Data Lake Analytics catalog assemblies.
"""

helps['dla catalog credential'] = """
type: group
short-summary: Manage Data Lake Analytics catalog credentials.
"""

helps['dla catalog credential create'] = """
type: command
short-summary: Create a new catalog credential for use with an external data source.
parameters:
  - name: --credential-name
    type: string
    short-summary: The name of the credential.
  - name: --database-name
    type: string
    short-summary: The name of the database in which to create the credential.
  - name: --user-name
    type: string
    short-summary: The user name that will be used when authenticating with this credential.
"""

helps['dla catalog credential delete'] = """
type: command
short-summary: Delete a catalog credential.
"""

helps['dla catalog credential list'] = """
type: command
short-summary: List catalog credentials.
"""

helps['dla catalog credential show'] = """
type: command
short-summary: Retrieve a catalog credential.
"""

helps['dla catalog credential update'] = """
type: command
short-summary: Update a catalog credential for use with an external data source.
parameters:
  - name: --credential-name
    type: string
    short-summary: The name of the credential to update.
  - name: --database-name
    type: string
    short-summary: The name of the database in which the credential exists.
  - name: --user-name
    type: string
    short-summary: The user name associated with the credential that will have its password updated.
"""

helps['dla catalog database'] = """
type: group
short-summary: Manage Data Lake Analytics catalog databases.
"""

helps['dla catalog external-data-source'] = """
type: group
short-summary: Manage Data Lake Analytics catalog external data sources.
"""

helps['dla catalog package'] = """
type: group
short-summary: Manage Data Lake Analytics catalog packages.
"""

helps['dla catalog procedure'] = """
type: group
short-summary: Manage Data Lake Analytics catalog stored procedures.
"""

helps['dla catalog schema'] = """
type: group
short-summary: Manage Data Lake Analytics catalog schemas.
"""

helps['dla catalog table'] = """
type: group
short-summary: Manage Data Lake Analytics catalog tables.
"""

helps['dla catalog table list'] = """
type: command
short-summary: List tables in a database or schema.
parameters:
  - name: --database-name
    type: string
    short-summary: The name of the database.
  - name: --schema-name
    type: string
    short-summary: The schema assocated with the tables to list.
"""

helps['dla catalog table-partition'] = """
type: group
short-summary: Manage Data Lake Analytics catalog table partitions.
"""

helps['dla catalog table-stats'] = """
type: group
short-summary: Manage Data Lake Analytics catalog table statistics.
"""

helps['dla catalog table-stats list'] = """
type: command
short-summary: List table statistics in a database, table, or schema.
parameters:
  - name: --database-name
    type: string
    short-summary: The name of the database.
  - name: --schema-name
    type: string
    short-summary: The schema associated with the tables to list.
  - name: --table-name
    type: string
    short-summary: The table to list statistics for. `--schema-name` must also be specified.
"""

helps['dla catalog table-type'] = """
type: group
short-summary: Manage Data Lake Analytics catalog table types.
"""

helps['dla catalog tvf'] = """
type: group
short-summary: Manage Data Lake Analytics catalog table valued functions.
"""

helps['dla catalog tvf list'] = """
type: command
short-summary: List table valued functions in a database or schema.
parameters:
  - name: --database-name
    type: string
    short-summary: The name of the database.
  - name: --schema-name
    type: string
    short-summary: The name of the schema assocated with table valued functions to list.
"""

helps['dla catalog view'] = """
type: group
short-summary: Manage Data Lake Analytics catalog views.
"""

helps['dla catalog view list'] = """
type: command
short-summary: List views in a database or schema.
parameters:
  - name: --database-name
    type: string
    short-summary: The name of the database.
  - name: --schema-name
    type: string
    short-summary: The name of the schema associated with the views to list.
"""

helps['dla job'] = """
type: group
short-summary: Manage Data Lake Analytics jobs.
"""

helps['dla job cancel'] = """
type: command
short-summary: Cancel a Data Lake Analytics job.
"""

helps['dla job list'] = """
type: command
short-summary: List Data Lake Analytics jobs.
"""

helps['dla job pipeline'] = """
type: group
short-summary: Manage Data Lake Analytics job pipelines.
"""

helps['dla job pipeline list'] = """
type: command
short-summary: List job pipelines in a Data Lake Analytics account.
"""

helps['dla job pipeline show'] = """
type: command
short-summary: Retrieve a job pipeline in a Data Lake Analytics account.
"""

helps['dla job recurrence'] = """
type: group
short-summary: Manage Data Lake Analytics job recurrences.
"""

helps['dla job recurrence list'] = """
type: command
short-summary: List job recurrences in a Data Lake Analytics account.
"""

helps['dla job recurrence show'] = """
type: command
short-summary: Retrieve a job recurrence in a Data Lake Analytics account.
"""

helps['dla job show'] = """
type: command
short-summary: Get information for a Data Lake Analytics job.
"""

helps['dla job submit'] = """
type: command
short-summary: Submit a job to a Data Lake Analytics account.
parameters:
  - name: --job-name
    type: string
    short-summary: Name for the submitted job.
  - name: --script
    type: string
    short-summary: Script to submit. This may be '@{file}' to load from a file.
  - name: --runtime-version
    short-summary: The runtime version to use.
    long-summary: This parameter is used for explicitly overwriting the default runtime. It should only be done if you know what you are doing.
  - name: --degree-of-parallelism
    short-summary: The degree of parallelism for the job.
    long-summary: Higher values equate to more parallelism and will usually yield faster running jobs, at the cost of more AUs.
  - name: --priority
    short-summary: The priority of the job.
    long-summary: Lower values increase the priority, with the lowest value being 1. This determines the order jobs are run in.
"""

helps['dla job wait'] = """
type: command
short-summary: Wait for a Data Lake Analytics job to finish.
long-summary: This command exits when the job completes.
parameters:
  - name: --job-id
    type: string
    short-summary: 'Job ID to poll for completion.'
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def network_client_factory(cli_ctx, **kwargs):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK, **kwargs)


def resource_client_factory(cli_ctx, **_):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)


def cf_application_gateways(cli_ctx, _):
    return network_client_factory(cli_ctx).application_gateways


def cf_app_gateway_waf_policy(cli_ctx, _):
    return network_client_factory(cli_ctx).web_application_firewall_policies


def cf_connection_monitor(cli_ctx, _):
    return network_client_factory(cli_ctx).connection_monitors


def cf_flow_logs(cli_ctx, _):
    return network_client_factory(cli_ctx).flow_logs


def cf_ddos_protection_plans(cli_ctx, _):
    return network_client_factory(cli_ctx).ddos_protection_plans


def cf_express_route_circuit_authorizations(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_circuit_authorizations


def cf_express_route_circuit_connections(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_circuit_connections


def cf_peer_express_route_circuit_connections(cli_ctx, _):
    return network_client_factory(cli_ctx).peer_express_route_circuit_connections


def cf_express_route_circuit_peerings(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_circuit_peerings


def cf_express_route_circuits(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_circuits


def cf_express_route_service_providers(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_service_providers


def cf_express_route_connections(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_connections


def cf_express_route_gateways(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_gateways


def cf_express_route_ports(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_ports


def cf_express_route_port_locations(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_ports_locations


def cf_express_route_links(cli_ctx, _):
    return network_client_factory(cli_ctx).express_route_links


def cf_private_endpoints(cli_ctx, _):
    return network_client_factory(cli_ctx).private_endpoints


def cf_private_dns_zone_groups(cli_ctx, _):
    return network_client_factory(cli_ctx).private_dns_zone_groups


def cf_private_endpoint_types(cli_ctx, _):
    return network_client_factory(cli_ctx).available_private_endpoint_types


def cf_private_link_services(cli_ctx, _):
    return network_client_factory(cli_ctx).private_link_services


def cf_load_balancers(cli_ctx, _):
    return network_client_factory(cli_ctx).load_balancers


def cf_load_balancer_backend_pools(cli_ctx, _):
    return network_client_factory(cli_ctx).load_balancer_backend_address_pools


def cf_local_network_gateways(cli_ctx, _):
    return network_client_factory(cli_ctx).local_network_gateways


def cf_network_interfaces(cli_ctx, _):
    return network_client_factory(cli_ctx).network_interfaces


def cf_network_profiles(cli_ctx, _):
    return network_client_factory(cli_ctx).network_profiles


def cf_network_security_groups(cli_ctx, _):
    return network_client_factory(cli_ctx).network_security_groups


def cf_network_watcher(cli_ctx, _):
    return network_client_factory(cli_ctx).network_watchers


def cf_packet_capture(cli_ctx, _):
    return network_client_factory(cli_ctx).packet_captures


def cf_private_access(cli_ctx, _):
    return network_client_factory(cli_ctx).available_private_access_services


def cf_public_ip_addresses(cli_ctx, _):
    return network_client_factory(cli_ctx).public_ip_addresses


def cf_public_ip_prefixes(cli_ctx, _):
    return network_client_factory(cli_ctx).public_ip_prefixes


def cf_security_rules(cli_ctx, _):
    return network_client_factory(cli_ctx).security_rules


def cf_service_tags(cli_ctx, _):
    return network_client_factory(cli_ctx).service_tags


def cf_subnets(cli_ctx, _):
    return network_client_factory(cli_ctx).subnets


def cf_usages(cli_ctx, _):
    return network_client_factory(cli_ctx).usages


def cf_virtual_network_gateway_connections(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_network_gateway_connections


def cf_virtual_network_gateways(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_network_gateways


def cf_virtual_networks(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_networks


def cf_virtual_network_peerings(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_network_peerings


def cf_service_aliases(cli_ctx, _):
    return network_client_factory(cli_ctx).available_service_aliases


def cf_dns_references(cli_ctx, _):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK_DNS).dns_resource_reference


def cf_dns_mgmt_zones(cli_ctx, _):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK_DNS).zones


def cf_dns_mgmt_record_sets(cli_ctx, _):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK_DNS).record_sets


def cf_virtual_router(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_routers


def cf_virtual_hub(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_hubs


def cf_virtual_router_peering(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_router_peerings


def cf_bastion_hosts(cli_ctx, _):
    return network_client_factory(cli_ctx).bastion_hosts


def cf_custom_ip_prefixes(cli_ctx, _):
    return network_client_factory(cli_ctx).custom_ip_prefixes



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from knack.log import get_logger
from azure.cli.core.util import sdk_no_wait
from azure.cli.core.azclierror import UnrecognizedArgumentError

logger = get_logger(__name__)


def _get_resource_group_location(cli_ctx, resource_group_name):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.cli.core.profiles import ResourceType
    client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)
    # pylint: disable=no-member
    return client.resource_groups.get(resource_group_name).location


def create_search_service(cmd, resource_group_name, search_service_name, sku, location=None, partition_count=0,
                          replica_count=0, public_network_access="enabled", ip_rules=None, identity_type=None,
                          hosting_mode="default", no_wait=False):
    """
    Creates a Search service in the given resource group.

    :param resource_group_name: Name of resource group.
    :param search_service_name: Name of the search service.
    :param sku: The SKU of the search service, which determines price tier and capacity limits.
    :param location: Geographic location of the resource.
    :param partition_count: Number of partitions in the search service.
    :param replica_count: Number of replicas in the search service.
    :param public_network_access: Public accessibility to the search service;
                                  allowed values are "enabled" or "disabled".
    :param ip_rules: Public IP(v4) addresses or CIDR ranges to the search service, seperated by comma or semicolon;
                     these IP rules are applicable only when --public-network-access is "enabled".
    :param identity_type: The identity type; possible values include: "None", "SystemAssigned".
    :param hosting_mode: The hosting mode; possible values include: "default", "highDensity";
                     Note that "highDensity" is only applicable to the standard3 SKU.
    """
    from azure.mgmt.search.models import SearchService, Sku, NetworkRuleSet, IpRule, Identity
    from azure.cli.command_modules.search._client_factory import cf_search_services
    import re

    _client = cf_search_services(cmd.cli_ctx, None)
    if location is None:
        location = _get_resource_group_location(cmd.cli_ctx, resource_group_name)

    _search = SearchService(location=location, sku=Sku(name=sku.lower()))

    replica_count = int(replica_count)
    partition_count = int(partition_count)
    if replica_count > 0:
        _search.replica_count = replica_count
    if partition_count > 0:
        _search.partition_count = partition_count
    if (public_network_access.lower() not in ["enabled", "disabled"]):
        raise UnrecognizedArgumentError(
            "SearchService.PublicNetworkAccess: only [""enabled"", ""disabled""] are allowed")
    if (hosting_mode not in ["default", "highDensity"]):
        raise UnrecognizedArgumentError(
            "SearchService.HostingMode: only [""default"", ""highDensity""] are allowed")
    if (hosting_mode == "highDensity" and sku.lower() != "standard3"):
        raise UnrecognizedArgumentError(
            "SearchService.HostingMode: ""highDensity"" is only allowed when sku is ""standard3""")
    _search.public_network_access = public_network_access
    _search.hosting_mode = hosting_mode

    if ip_rules:
        _ip_rules = []
        _ip_rules_array = re.split(';|,', ip_rules)
        for _ip_rule in _ip_rules_array:
            if _ip_rule:
                _ip_rules.append(IpRule(value=_ip_rule))
        _search.network_rule_set = NetworkRuleSet(ip_rules=_ip_rules)
    if identity_type:
        _identity = Identity(type=identity_type)
        _search.identity = _identity

    return sdk_no_wait(no_wait, _client.begin_create_or_update, resource_group_name, search_service_name, _search)


def update_search_service(instance, partition_count=0, replica_count=0, public_network_access=None,
                          ip_rules=None, identity_type=None):
    """
    Update partition and replica of the given search service.

    :param partition_count: Number of partitions in the search service.
    :param replica_count: Number of replicas in the search service.
    :param public_network_access: Public accessibility to the search service;
                                  allowed values are "enabled" or "disabled".
    :param ip_rules: Public IP(v4) addresses or CIDR ranges to the search service, seperated by comma(',') or
                     semicolon(';'); If spaces (' '), ',' or ';' is provided, any existing IP rule will be
                     nullified and no public IP rule is applied. These IP rules are applicable only when
                     public_network_access is "enabled".
    :param identity_type: The identity type; possible values include: "None", "SystemAssigned".
    """
    from azure.mgmt.search.models import NetworkRuleSet, IpRule, Identity
    import re

    replica_count = int(replica_count)
    partition_count = int(partition_count)
    if replica_count > 0:
        instance.replica_count = replica_count
    if partition_count > 0:
        instance.partition_count = partition_count
    if public_network_access:
        if (public_network_access.lower() not in ["enabled", "disabled"]):
            raise UnrecognizedArgumentError(
                "SearchService.PublicNetworkAccess: only [""enabled"", ""disabled""] are allowed")
        instance.public_network_access = public_network_access
    if ip_rules:
        _ip_rules = []
        _ip_rules_array = re.split(';|,', ip_rules)
        for _ip_rule in _ip_rules_array:
            if _ip_rule:
                _ip_rules.append(IpRule(value=_ip_rule))
        instance.network_rule_set = NetworkRuleSet(ip_rules=_ip_rules)
    if identity_type:
        _identity = Identity(type=identity_type)
        instance.identity = _identity

    return instance


def update_private_endpoint_connection(cmd, resource_group_name, search_service_name, private_endpoint_connection_name,
                                       private_link_service_connection_status,
                                       private_link_service_connection_description,
                                       private_link_service_connection_actions_required):
    """
    Update an existing private endpoint connection in a Search service in the given resource group.

    :param resource_group_name: Name of resource group.
    :param search_service_name: Name of the search service.
    :param private_endpoint_connection_name: Name of the private endpoint connection resource;
        for example: {the name of the private endpoint resource}.{guid}.
    :param private_link_service_connection_status: The updated status of the private endpoint connection resource.
        Possible values include: "Pending", "Approved", "Rejected", "Disconnected".
    :param private_link_service_connection_description: Custom description when updating
        the private endpoint connection resource.
    :param private_link_service_connection_actions_required: Custom 'actions required' message when updating
        the private endpoint connection resource.
    """

    from azure.mgmt.search.models import PrivateEndpointConnection, PrivateEndpointConnectionProperties,\
        PrivateEndpointConnectionPropertiesPrivateLinkServiceConnectionState
    from azure.cli.command_modules.search._client_factory import cf_search_private_endpoint_connections

    _client = cf_search_private_endpoint_connections(cmd.cli_ctx, None)
    _private_endpoint_connection = PrivateEndpointConnection()
    _private_link_service_connection_state = PrivateEndpointConnectionPropertiesPrivateLinkServiceConnectionState(
        status=private_link_service_connection_status,
        description=private_link_service_connection_description,
        actions_required=private_link_service_connection_actions_required
    )
    _private_endpoint_connection_properties = PrivateEndpointConnectionProperties(
        private_link_service_connection_state=_private_link_service_connection_state)
    _private_endpoint_connection.id = private_endpoint_connection_name
    _private_endpoint_connection.properties = _private_endpoint_connection_properties

    return _client.update(resource_group_name, search_service_name, private_endpoint_connection_name,
                          _private_endpoint_connection)


def create_shared_private_link_resource(cmd, resource_group_name, search_service_name,
                                        shared_private_link_resource_name, shared_private_link_resource_id,
                                        shared_private_link_resource_group_id,
                                        shared_private_link_resource_request_message="Please approve",
                                        no_wait=False):
    """
    Create shared privatelink resources in a Search service in the given resource group.

    :param resource_group_name: Name of resource group.
    :param search_service_name: Name of the search service.
    :param shared_private_link_resource_name: Name of the shared private link resource.
    :param shared_private_link_resource_id: Fully qualified resource ID for the resource.
        for example: /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/
        {resourceProviderNamespace}/{resourceType}/{resourceName}.
    :param shared_private_link_resource_group_id: The group id of the resource; for example: blob, sql or vault.
    :param shared_private_link_resource_request_message: Custom request message when creating or updating the shared
        privatelink resources.
    """
    from azure.mgmt.search.models import SharedPrivateLinkResource, SharedPrivateLinkResourceProperties
    from azure.cli.command_modules.search._client_factory import cf_search_shared_private_link_resources

    _client = cf_search_shared_private_link_resources(cmd.cli_ctx, None)
    _shared_private_link_resource = SharedPrivateLinkResource()
    _shared_private_link_resource.name = shared_private_link_resource_name
    _shared_private_link_resource.properties = SharedPrivateLinkResourceProperties(
        private_link_resource_id=shared_private_link_resource_id,
        group_id=shared_private_link_resource_group_id,
        request_message=shared_private_link_resource_request_message
    )

    return sdk_no_wait(no_wait, _client.begin_create_or_update, resource_group_name,
                       search_service_name, shared_private_link_resource_name, _shared_private_link_resource)


def update_shared_private_link_resource(cmd, resource_group_name, search_service_name,
                                        shared_private_link_resource_name, shared_private_link_resource_id,
                                        shared_private_link_resource_group_id,
                                        shared_private_link_resource_request_message,
                                        no_wait=False):
    """
    Update shared privatelink resources in a Search service in the given resource group.

    :param resource_group_name: Name of resource group.
    :param search_service_name: Name of the search service.
    :param shared_private_link_resource_name: Name of the shared private link resource.
    :param shared_private_link_resource_id: Fully qualified resource ID for the resource;
        for example: /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/
        {resourceProviderNamespace}/{resourceType}/{resourceName}.
    :param shared_private_link_resource_group_id: The group id of the resource; for example: blob, sql or vault.
    :param shared_private_link_resource_request_message: Custom request message when creating or updating the shared
        privatelink resources.
    """
    from azure.mgmt.search.models import SharedPrivateLinkResource, SharedPrivateLinkResourceProperties
    from azure.cli.command_modules.search._client_factory import cf_search_shared_private_link_resources

    _client = cf_search_shared_private_link_resources(cmd.cli_ctx, None)
    _shared_private_link_resource = SharedPrivateLinkResource()
    _shared_private_link_resource.name = shared_private_link_resource_name
    _shared_private_link_resource.properties = SharedPrivateLinkResourceProperties(
        private_link_resource_id=shared_private_link_resource_id,
        group_id=shared_private_link_resource_group_id,
        request_message=shared_private_link_resource_request_message
    )

    return sdk_no_wait(no_wait, _client.begin_create_or_update, resource_group_name,
                       search_service_name, shared_private_link_resource_name, _shared_private_link_resource)
